{% load i18n %}

<div class="table-responsive">
    <table class="table table-striped table-width-fix tab-contract" id="{{ tab_name }}">
        <thead>
            <tr>
                <th>{% translate "Status" %}</th>
                <th>{% translate "Start" %}</th>
                <th>{% translate "Destination" %}</th>
                <th>{% translate "Reward ISK" %}</th>
                <th>{% translate "Collateral ISK" %}</th>
                <th>{% translate "Volume m3" %}</th>
                <th>{% translate "Contract Check" %}</th>
                <th>{% translate "Issued On" %}</th>
                <th>{% translate "Expires On" %}</th>
                <th>{% translate "Issued By" %}</th>
                <th>{% translate "Accepted On" %}</th>
                <th>{% translate "Accepted By" %}</th>
            </tr>
        </thead>
        <tbody>
        </tbody>
    </table>
</div>
